
//This is the Arduino sketch for my Tintin XFLR-6 Rocket build. With a single button press, the launch sequence goes through several stages.
//The first press turns on the green LED lights and plays the Tintin theme song. The second press arms the launch sequence, which turns the red
//LED in the button on, changes the green lights to red, and turns the digits on for the countdown. The third button press starts the countdown timer,
//plays the countdown and rocket ignition audio and starts the red LEDs to fade on and off to simulate engines firing. 

//PIN CHECKLIST

// D2 = Seven Segment A
// D3 = Fading Red LEDs - it needs to be a PWM able digital pin on nano!
// D4 = Seven Segment C
// D5 = Seven Segment D
// D6 = Seven Segment E
// D7 = Seven Segment F
// D8 = Seven Segment G
// D9 = Speaker - don't change this pin if you use a nano!
// D10 = CS on SD Card - don't change this pin if you use a nano!
// D11 = MOSI on SD Card - don't change this pin if you use a nano!
// D12 = MISO on SD Card - don't change this pin if you use a nano!
// D13 = SCK on SD Card - don't change this pin if you use a nano!

// A0 = Countdown Digits Ones
// A1 = Coutdown Digits Tens
// A2 = Main Button
// A3 = Blinking Red LEDs
// A4 = Blinking Green LEDs
// A5 = Seven Segment B

//A6 and A7 are analog input only on nano!


// LIBRARIES ////////////////////////////////////////////////////////////////////////////////////////// 

#include <SD.h> // need to include the SD library
#include <TMRpcm.h> //Arduino library for asynchronous playback of PCM/WAV files
#include <SPI.h> //  need to include the SPI library

// DEFINITIONS //////////////////////////////////////////////////////////////////////////////////////////

//define Chip Select (CS) pin of SD card

    #define SD_ChipSelectPin 10 //must be pin D10 on nano!!
    
// define directions for blastRED LED fade
    #define UP 0
    #define DOWN 1

    char tintin[] = "tintin.wav";
    char rocket[] = "rocket.wav";

// CONSTANTS /////////////////////////////////////////////////////////////////////////////////////////////

// display segment pins (note: no pin for decimmal point assigned!)

    const int SEG_a = 2;    //segment A is set to pin D2
    const int SEG_b = A5;   //segment B is set to pin D3
    const int SEG_c = 4;    //segment C is set to pin A4
    const int SEG_d = 5;    //segment D is set to pin D5
    const int SEG_e = 6;    //segment E is set to pin D6
    const int SEG_f = 7;    //segment F is set to pin D7
    const int SEG_g = 8;    //segment G is set to pin D8  

// digit pins
    
    const int DIG_ones = A0;      //the pin to display the ones digit
    const int DIG_tens = A1;      //the pin to display the tens digit

// button pin
    
    const int button = A2;        //main button 

// LED lights pins

    const int blinkRED = A3;      //blinking red LEDs set to pin A3
    const int blinkGREEN = A4;     //blinking green LEDs set to pin A4

    const int blastRED = 3;      //fading red rocket engine LED set to pin 3 for PWM

// blinking LEDs

    const int blinkGREEN_interval = 2000;
    const int blinkRED_interval = 1000;
    const int blinkDuration = 500; // number of millisecs that all LEDs are on   
    const int buttonInterval = 300; // number of millisecs between button readings

// fading LED
    const int minPWM = 0;   // constants for min and max PWM
    const int maxPWM = 255;

// State Variable for Fade Direction
    byte fadeDirection = UP;

// Global Fade Value
    int fadeValue = 0;                    // but be bigger than byte and signed, for rollover
    byte fadeIncrement = 5;               // How smooth to fade?
    unsigned long previousFadeMillis;     // millis() timing Variable, just for fading
    int fadeInterval = 50;                // How fast to increment?    

// VARIABLES /////////////////////////////////////////////////////////////////////////////////////////////

    int timee = 0;          //the countdown time
    int tens = 0;           //the numbers in the tens digit                        
    int ones = 0;           //the numbers in the ones digit

    int val;                // variable for reading the pin status
    int val2;               // variable for reading the delayed status
    int buttonState;        // variable to hold the button state
    int Mode = 0;           // What mode is the launch program in?
 
    byte blinkGREEN_state = LOW;           //   LOW = off
    byte blinkRED_state = LOW;
        
    unsigned long currentMillis = 0;    // stores the value of millis() in each iteration of loop()
    unsigned long previous_blinkGREEN_millis = 0;
    unsigned long previous_blinkRED_millis = 0; 
    unsigned long time_countdown = 0;

    int playedTintin = 0; //counter to make sure the Tintin theme song audio doesn't loop and create feedback
    int playedRocket = 0; //counter to make sure the countdown/rocket launch audio doesn't loop and create feedback


// OBJECTS ////////////////////////////////////////////////////////////////////////////////////////////

    TMRpcm tmrpcm; // create an object for use in this sketch

// MAIN SETUP ////////////////////////////////////////////////////////////////////////////////////////////
  
    void setup() {

// debug serial
    
    Serial.begin(9600);       //serial communication opened for debugging
   if (!SD.begin(SD_ChipSelectPin)) // returns 1 if the card is present
   {
      Serial.println("SD fail");
      return;
    }

 
// outputs  
    
    pinMode(SEG_a, OUTPUT);   //all seven segments set to output
    pinMode(SEG_b, OUTPUT);
    pinMode(SEG_c, OUTPUT);
    pinMode(SEG_d, OUTPUT);
    pinMode(SEG_e, OUTPUT);
    pinMode(SEG_f, OUTPUT);
    pinMode(SEG_g, OUTPUT);
    
    pinMode(DIG_tens, OUTPUT);  //both digits set to output
    pinMode(DIG_ones, OUTPUT);

    pinMode(blinkRED, OUTPUT);    //all LEDs set to output
    pinMode(blinkGREEN, OUTPUT);
    pinMode(blastRED, OUTPUT);

    tmrpcm.speakerPin = 9; //pin for speaker - don't change this on nano!
    tmrpcm.setVolume(5); //volume for speaker
    
// inputs
    
    pinMode(button, INPUT_PULLUP);   // Set the button pin as input

// states
    
    buttonState = digitalRead(button);    // read the initial state of the button
    analogWrite(blastRED, fadeValue);       // put pwmLED into known state (off)
 
    }


// FUNCTIONS /////////////////////////////////////////////////////////////////////////////////////////////

// this function assigns segments to dislay numerals from 0-9

  void display_number(int number) {
  switch (number) {
    case 0: //number zero
      digitalWrite(SEG_a, HIGH);
      digitalWrite(SEG_b, HIGH);
      digitalWrite(SEG_c, HIGH);
      digitalWrite(SEG_d, HIGH);
      digitalWrite(SEG_e, HIGH);
      digitalWrite(SEG_f, HIGH);
      digitalWrite(SEG_g, LOW);
      break;
    case 1: //number one
      digitalWrite(SEG_a, LOW);
      digitalWrite(SEG_b, HIGH);
      digitalWrite(SEG_c, HIGH);
      digitalWrite(SEG_d, LOW);
      digitalWrite(SEG_e, LOW);
      digitalWrite(SEG_f, LOW);
      digitalWrite(SEG_g, LOW);
      break;
    case 2: //number two
      digitalWrite(SEG_a, HIGH);
      digitalWrite(SEG_b, HIGH);
      digitalWrite(SEG_c, LOW);
      digitalWrite(SEG_d, HIGH);
      digitalWrite(SEG_e, HIGH);
      digitalWrite(SEG_f, LOW);
      digitalWrite(SEG_g, HIGH);
      break;
    case 3: //number three
      digitalWrite(SEG_a, HIGH);
      digitalWrite(SEG_b, HIGH);
      digitalWrite(SEG_c, HIGH);
      digitalWrite(SEG_d, HIGH);
      digitalWrite(SEG_e, LOW);
      digitalWrite(SEG_f, LOW);
      digitalWrite(SEG_g, HIGH);
      break;
    case 4: //number four
      digitalWrite(SEG_a, LOW);
      digitalWrite(SEG_b, HIGH);
      digitalWrite(SEG_c, HIGH);
      digitalWrite(SEG_d, LOW);
      digitalWrite(SEG_e, LOW);
      digitalWrite(SEG_f, HIGH);
      digitalWrite(SEG_g, HIGH);
      break;
    case 5: //number five
      digitalWrite(SEG_a, HIGH);
      digitalWrite(SEG_b, LOW);
      digitalWrite(SEG_c, HIGH);
      digitalWrite(SEG_d, HIGH);
      digitalWrite(SEG_e, LOW);
      digitalWrite(SEG_f, HIGH);
      digitalWrite(SEG_g, HIGH);
      break;
    case 6: //number six
      digitalWrite(SEG_a, HIGH);
      digitalWrite(SEG_b, LOW);
      digitalWrite(SEG_c, HIGH);
      digitalWrite(SEG_d, HIGH);
      digitalWrite(SEG_e, HIGH);
      digitalWrite(SEG_f, HIGH);
      digitalWrite(SEG_g, HIGH);
      break;
    case 7: //number seven
      digitalWrite(SEG_a, HIGH);
      digitalWrite(SEG_b, HIGH);
      digitalWrite(SEG_c, HIGH);
      digitalWrite(SEG_d, LOW);
      digitalWrite(SEG_e, LOW);
      digitalWrite(SEG_f, LOW);
      digitalWrite(SEG_g, LOW);
      break;
    case 8: //number eight
      digitalWrite(SEG_a, HIGH);
      digitalWrite(SEG_b, HIGH);
      digitalWrite(SEG_c, HIGH);
      digitalWrite(SEG_d, HIGH);
      digitalWrite(SEG_e, HIGH);
      digitalWrite(SEG_f, HIGH);
      digitalWrite(SEG_g, HIGH);
      break;
    case 9: //number nine
      digitalWrite(SEG_a, HIGH);
      digitalWrite(SEG_b, HIGH);
      digitalWrite(SEG_c, HIGH);
      digitalWrite(SEG_d, HIGH);
      digitalWrite(SEG_e, LOW);
      digitalWrite(SEG_f, HIGH);
      digitalWrite(SEG_g, HIGH);
      break;
  }
}

// this function chooses which digit to multiplex the numbers to
    
    void pick_digit(int digit) {
      digitalWrite(DIG_tens, HIGH);
      digitalWrite(DIG_ones, HIGH);

      switch(digit) {
        case 1: digitalWrite(DIG_tens, LOW); break;
        case 2: digitalWrite(DIG_ones, LOW); break;
      }
}

// this function slots the digits from the countdown into the correct digit
    
    void time_display(int timee) {
        unsigned char tens = int (timee/10)%10;
        unsigned char ones = int (timee%10);
      
        display_number(tens);
        pick_digit(2);
        delay(5);

        display_number(ones);
        pick_digit(1);
        delay(5);
}


// this function makes the red rocket engine LED fade on and off without using the delay

    void fadingRED(unsigned long thisMillis) {
  
        // is it time to update yet?
        // if not, nothing happens
        if (thisMillis - previousFadeMillis >= fadeInterval) {
          // yup, it's time!
          if (fadeDirection == UP) {
              fadeValue = fadeValue + fadeIncrement;  
              if (fadeValue >= maxPWM) {
              // At max, limit and change direction
              fadeValue = maxPWM;
              fadeDirection = DOWN;
              }
          } 
          else {
          //if we aren't going up, we're going down
              fadeValue = fadeValue - fadeIncrement;
                if (fadeValue <= minPWM) {
                // At min, limit and change direction
                fadeValue = minPWM;
                fadeDirection = UP;
                }
           }
          // Only need to update when it changes
          analogWrite(blastRED, fadeValue);  
 
          // reset millis for the next iteration (fade timer only)
          previousFadeMillis = thisMillis;
        }
    }
    


// this function starts the timer countdown

    void countdown(){
      currentMillis = millis();
      time_countdown = millis();

        if (playedRocket == 0) { // checks if the rocket.wav file hasn't played
          tmrpcm.play(rocket);   // if not, go ahead and play it
          playedRocket++;        // stop the feedback loop by changing the int to >0
        }   

        while (timee > 0) {
          time_display(timee);
      
            if (millis() - time_countdown >= 998) { //The while loop ends when 'timee' become zero
                timee = timee - 1;
                time_countdown = millis();
             }
      
            if (timee == 0) { //if time arrive to zero the display "00" and then flashes "XX" twice
                
                digitalWrite(DIG_tens, HIGH);
                digitalWrite(DIG_ones, HIGH);
                digitalWrite(SEG_a, HIGH);
                digitalWrite(SEG_b, HIGH);
                digitalWrite(SEG_c, HIGH);
                digitalWrite(SEG_d, HIGH);
                digitalWrite(SEG_e, HIGH);
                digitalWrite(SEG_f, HIGH);
                digitalWrite(SEG_g, LOW);
                delay(1000);

                digitalWrite(DIG_tens, HIGH);
                digitalWrite(DIG_ones, HIGH);
                digitalWrite(SEG_a, LOW);
                digitalWrite(SEG_b, HIGH);
                digitalWrite(SEG_c, HIGH);
                digitalWrite(SEG_d, LOW);
                digitalWrite(SEG_e, HIGH);
                digitalWrite(SEG_f, HIGH);
                digitalWrite(SEG_g, HIGH);
                delay(100);
                
                digitalWrite(DIG_tens, LOW);
                digitalWrite(DIG_ones, LOW); 
                delay(100);
                
                digitalWrite(DIG_tens, HIGH);
                digitalWrite(DIG_ones, HIGH);
                digitalWrite(SEG_a, LOW);
                digitalWrite(SEG_b, HIGH);
                digitalWrite(SEG_c, HIGH);
                digitalWrite(SEG_d, LOW);
                digitalWrite(SEG_e, HIGH);
                digitalWrite(SEG_f, HIGH);
                digitalWrite(SEG_g, HIGH);
                delay(100);
                
                digitalWrite(DIG_tens, LOW);
                digitalWrite(DIG_ones, LOW); 
                }
          }

        }
 


// this function makes the green LED blink

    void blinkingGREEN() {

        if (blinkGREEN_state == LOW) {
          if (currentMillis - previous_blinkGREEN_millis >= blinkGREEN_interval) {
              blinkGREEN_state = HIGH;
              previous_blinkGREEN_millis += blinkGREEN_interval;
          }
        }
        else {
          if (currentMillis - previous_blinkGREEN_millis >= blinkDuration) {
              blinkGREEN_state = LOW;
              previous_blinkGREEN_millis += blinkDuration;
          } 
        }
        digitalWrite(blinkGREEN, blinkGREEN_state);    
    }

// this function makes the red LED blink

    void blinkingRED() {
      
      if (blinkRED_state == LOW) {
        if (currentMillis - previous_blinkRED_millis >= blinkRED_interval) {
            blinkRED_state = HIGH;
            previous_blinkRED_millis += blinkRED_interval;
        }
      }
      else {
        if (currentMillis - previous_blinkRED_millis >= blinkDuration) {
            blinkRED_state = LOW;
            previous_blinkRED_millis += blinkDuration;
        } 
      }
      digitalWrite(blinkRED, blinkRED_state);    
    }


// this function checks to see if the Tintin theme song has been played, and if not, plays it. This keeps it from endlessly looping during the button Mode 1

void playTintin(){

if (playedTintin == 0) {
    tmrpcm.play(tintin);
    playedTintin++;
    }    
}


// this function resets the audio playback to zero in button Mode 0

void clearAudio(){
    tmrpcm.stopPlayback();
    playedTintin = 0;
    playedRocket = 0;
}




//MAIN LOOP/////////////////////////////////////////////////////////////////////////////////////////////

    void loop(){

        currentMillis = millis();

        val = digitalRead(button);        // read input value and store it in val
        delay(10);                        // 10 milliseconds is a good amount of time to wait
        val2 = digitalRead(button);       // read the input again to check for bounces
        if (val == val2) {                // make sure we got 2 consistant readings!
          if (val != buttonState) {       // the button state has changed!
          if (val == LOW) {               // check if the button is pressed
            if (Mode == 0) {          
              Mode = 1;               
            } else {
                if (Mode == 1) {        
                Mode = 2;           
            } else {
                if (Mode == 2) {      
                Mode = 3;           
            } else {
                if (Mode == 3) { 
                Mode = 0;          
                  }
            }
           }
          }
         }
        }
        buttonState = val;                 // save the new state in our variable
      }

      // choose the various modes
      if (Mode == 0) { // reset mode: all audio and lights off
        clearAudio();
        digitalWrite(blinkGREEN, LOW);
        digitalWrite(blinkRED, LOW);
        digitalWrite(blastRED, LOW);
        digitalWrite(DIG_tens, LOW);
        digitalWrite(DIG_ones, LOW);
        timee = 0;
                 Serial.write("click1");
      }

      if (Mode == 1) { // basic mode: blinking green LED are on, and the Tintin theme song starts playing everything else off.
        blinkingGREEN();
        playTintin();
        digitalWrite(blinkRED, LOW);
        digitalWrite(blastRED, LOW);
                Serial.write("click2");
      }

      if (Mode == 2) { // arm mode: blinking green LEDs turn red and the countdown switch is armed (displays 10 seconds)       
        digitalWrite(blinkGREEN, LOW);
        digitalWrite(blinkRED, HIGH);
        digitalWrite(blastRED, LOW);
        timee = 10;
        time_display(timee);
                Serial.write("click3");
      }

      if (Mode == 3) { //countdown mode: audio countdown/blast off plays, red LEDs start blinking, timer countsdown and sets off red engine LED after reaching zero.
        digitalWrite(blinkGREEN, LOW);
        blinkingRED();    
        countdown();
        fadingRED(currentMillis);       
                Serial.write("click4");
      }    
    }
